@@ -16,13 +16,14 @@ const RENAMED_DESKTOP_IDS = {
     'glchess.desktop': 'gnome-chess.desktop',
     'glines.desktop': 'five-or-more.desktop',
     'gnect.desktop': 'four-in-a-row.desktop',
-    'gnibbles.desktop': 'gnome-nibbles.desktop',
+    'gnibbles.desktop': 'org.gnome.Nibbles.desktop',
     'gnobots2.desktop': 'gnome-robots.desktop',
     'gnome-boxes.desktop': 'org.gnome.Boxes.desktop',
     'gnome-clocks.desktop': 'org.gnome.clocks.desktop',
     'gnome-contacts.desktop': 'org.gnome.Contacts.desktop',
     'gnome-documents.desktop': 'org.gnome.Documents.desktop',
     'gnome-font-viewer.desktop': 'org.gnome.font-viewer.desktop',
+    'gnome-nibbles.desktop': 'org.gnome.Nibbles.desktop',
     'gnome-photos.desktop': 'org.gnome.Photos.desktop',
     'gnome-screenshot.desktop': 'org.gnome.Screenshot.desktop',
     'gnome-software.desktop': 'org.gnome.Software.desktop',
